@@ -823,7 +823,7 @@ static inline void discovery_share_cgroups_with_ebpf() {
     }
     sem_wait(shm_mutex_cgroup_ebpf);
 
-    for (cg = cgroup_root, count = 0; cg; cg = cg->next, count++) {
+    for (cg = cgroup_root, count = 0; cg && count < cgroup_root_max; cg = cg->next, count++) {
         netdata_ebpf_cgroup_shm_body_t *ptr = &shm_cgroup_ebpf.body[count];
         char *prefix = (is_cgroup_systemd_service(cg)) ? services_chart_id_prefix : cgroup_chart_id_prefix;
         snprintfz(ptr->name, CGROUP_EBPF_NAME_SHARED_LENGTH - 1, "%s%s", prefix, cg->chart_id);
@@ -843,6 +843,13 @@ static inline void discovery_share_cgroups_with_ebpf() {
         netdata_log_debug(D_CGROUP, "cgroup shared: NAME=%s, ENABLED=%d", ptr->name, ptr->enabled);
     }
 
+    if (unlikely(cg != NULL)) {
+        nd_log_limit_static_global_var(erl, 3600, 0);
+        nd_log_limit(&erl, NDLS_COLLECTORS, NDLP_WARNING,
+                     "CGROUP: shared memory buffer full (%d cgroups). Some cgroups were not shared with eBPF.",
+                     cgroup_root_max);
+    }
+
     shm_cgroup_ebpf.header->cgroup_root_count = count;
     sem_post(shm_mutex_cgroup_ebpf);
 }
