@@ -727,8 +727,19 @@ extent_flush_to_open(struct rrdengine_instance *ctx, struct extent_io_descriptor
 
 static bool datafile_is_full(struct rrdengine_instance *ctx, struct rrdengine_datafile *datafile) {
     bool ret = false;
+
     spinlock_lock(&datafile->writers.spinlock);
 
+#ifdef OS_WINDOWS
+    time_t now = now_realtime_sec();
+    if (now - datafile->writers.last_sync_time > 60) {
+        nd_log_daemon(NDLP_INFO, "DBENGINE: datafile %d, last sync time: %ld, now: %ld", datafile->fileno, datafile->writers.last_sync_time, now);
+        sync_uv_file_data(datafile->file);
+        sync_uv_file_data(datafile->journalfile->file);
+        datafile->writers.last_sync_time = now_realtime_sec();
+    }
+#endif
+
     if(datafile->pos > rrdeng_target_data_file_size(ctx))
         ret = true;
 
@@ -863,6 +874,12 @@ static struct rrdengine_datafile *get_datafile_to_write_extent(struct rrdengine_
         // release the writers on the old datafile
         spinlock_lock(&old_datafile->writers.spinlock);
         old_datafile->writers.running--;
+#ifdef OS_WINDOWS
+        sync_uv_file_data(old_datafile->file);
+        sync_uv_file_data(old_datafile->journalfile->file);
+        datafile->writers.last_sync_time = now_realtime_sec();
+        old_datafile->writers.last_sync_time = now_realtime_sec();
+#endif
         spinlock_unlock(&old_datafile->writers.spinlock);
     }
 
@@ -2301,10 +2318,8 @@ static inline void worker_dispatch_query_prep(struct rrdeng_cmd cmd, bool from_w
 uint64_t rrdeng_get_directory_free_bytes_space(struct rrdengine_instance *ctx)
 {
     uint64_t free_bytes = 0;
-    struct statvfs buff_statvfs;
-    if (statvfs(ctx->config.dbfiles_path, &buff_statvfs) == 0)
-        free_bytes = buff_statvfs.f_bavail * buff_statvfs.f_bsize;
-
+    OS_SYSTEM_DISK_SPACE space = os_disk_space(ctx->config.dbfiles_path);
+    free_bytes = OS_SYSTEM_DISK_SPACE_OK(space) ? space.free_bytes : 0;
     return (free_bytes - (free_bytes * 5 / 100));
 }
 
